"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.deleteJSON = exports.readJSON = exports.writeJSON = exports.toJSON = exports.fromJSON = exports.json = void 0;
// external
const errlop_1 = __importDefault(require("errlop"));
const file_1 = require("@bevry/file");
/** Help the JSON parser serialize the object to JSON string. */
function replacer(key, value) {
    if (value instanceof Set) {
        return {
            dataType: 'Set',
            value: Array.from(value.values()),
        };
    }
    else if (value instanceof Map) {
        return {
            dataType: 'Map',
            value: Array.from(value.entries()),
        };
    }
    else {
        return value;
    }
}
/** Help the JSON parser deserialize the JSON string. */
function reviver(key, value) {
    if (typeof value === 'object' && value !== null) {
        if (value.dataType === 'Set') {
            return new Set(value.value);
        }
        else if (value.dataType === 'Map') {
            return new Map(value.value);
        }
    }
    return value;
}
/** Clone/dereference a JSON object. */
function json(data) {
    return fromJSON(toJSON(data));
}
exports.json = json;
/** Deserialize a JSON object. */
function fromJSON(data) {
    try {
        return JSON.parse(data, reviver);
    }
    catch (err) {
        throw new errlop_1.default(`failed to deserialize the json `, err);
    }
}
exports.fromJSON = fromJSON;
/** Serialize a JSON object. */
function toJSON(data) {
    try {
        return JSON.stringify(data, replacer, '  ');
    }
    catch (err) {
        throw new errlop_1.default(`failed to serialize the json `, err);
    }
}
exports.toJSON = toJSON;
/** Write a JSON file that will be serialized with {@link toJSON}. */
async function writeJSON(path, data) {
    try {
        const contents = toJSON(data);
        await (0, file_1.writeFile)(path, contents);
    }
    catch (err) {
        throw new errlop_1.default(`failed to write the json for the file: ${path}`, err);
    }
}
exports.writeJSON = writeJSON;
/** Read a JSON file that was serialized with {@link toJSON}. */
async function readJSON(path) {
    try {
        const data = await (0, file_1.readFile)(path);
        return fromJSON(data);
    }
    catch (err) {
        throw new errlop_1.default(`failed to read the json for the file: ${path}`, err);
    }
}
exports.readJSON = readJSON;
/** Delete a JSON file. */
function deleteJSON(path) {
    return (0, file_1.deleteFile)(path);
}
exports.deleteJSON = deleteJSON;
